@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2024 Joel Rosdahl and other contributors
+// Copyright (C) 2020-2025 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -18,22 +18,21 @@
 
 #include "argprocessing.hpp"
 
-#include <ccache/Args.hpp>
-#include <ccache/ArgsInfo.hpp>
-#include <ccache/Context.hpp>
-#include <ccache/Depfile.hpp>
+#include <ccache/args.hpp>
+#include <ccache/argsinfo.hpp>
 #include <ccache/compopt.hpp>
+#include <ccache/context.hpp>
 #include <ccache/core/common.hpp>
+#include <ccache/depfile.hpp>
 #include <ccache/language.hpp>
-#include <ccache/util/DirEntry.hpp>
-#include <ccache/util/PathString.hpp>
-#include <ccache/util/Tokenizer.hpp>
 #include <ccache/util/assertions.hpp>
+#include <ccache/util/direntry.hpp>
 #include <ccache/util/filesystem.hpp>
 #include <ccache/util/format.hpp>
 #include <ccache/util/logging.hpp>
 #include <ccache/util/path.hpp>
 #include <ccache/util/string.hpp>
+#include <ccache/util/tokenizer.hpp>
 #include <ccache/util/wincompat.hpp>
 
 #ifdef HAVE_UNISTD_H
@@ -54,7 +53,6 @@ namespace fs = util::filesystem;
 
 using core::Statistic;
 using util::DirEntry;
-using pstr = util::PathString;
 
 namespace {
 
@@ -81,7 +79,7 @@ struct ArgumentProcessingState
   bool found_fpch_preprocess = false;
   bool found_Yu = false;
   bool found_Yc = false;
-  std::string found_Fp_file;
+  fs::path found_Fp_file;
   bool found_valid_Fp = false;
   bool found_syntax_only = false;
   ColorDiagnostics color_diagnostics = ColorDiagnostics::automatic;
@@ -159,7 +157,7 @@ detect_pch(const std::string& option,
   // If the option is an option for Clang (is_cc1_option), don't accept
   // anything just because it has a corresponding precompiled header,
   // because Clang doesn't behave that way either.
-  std::string pch_file;
+  fs::path pch_file;
   if (option == "-Yc") {
     state.found_Yc = true;
     args_info.generating_pch = true;
@@ -175,10 +173,10 @@ detect_pch(const std::string& option,
       pch_file = included_pch_file;
       included_pch_file.clear(); // reset pch file set from /Fp
     } else {
-      fs::path file = fs::path(arg).replace_extension(".pch");
+      fs::path file = util::with_extension(arg, ".pch");
       if (fs::is_regular_file(file)) {
         LOG("Detected use of precompiled header: {}", file);
-        pch_file = pstr(file).str();
+        pch_file = file;
       }
     }
   } else if (option == "-Fp") {
@@ -238,7 +236,7 @@ detect_pch(const std::string& option,
 bool
 process_profiling_option(const Context& ctx,
                          ArgsInfo& args_info,
-                         const std::string& arg)
+                         std::string_view arg)
 {
   static const std::vector<std::string> known_simple_options = {
     "-fprofile-correction",
@@ -256,7 +254,13 @@ process_profiling_option(const Context& ctx,
     return true;
   }
 
-  std::string new_profile_path;
+  if (util::starts_with(arg, "-fprofile-prefix-path=")) {
+    args_info.profile_prefix_path = arg.substr(arg.find('=') + 1);
+    LOG("Set profile prefix path to {}", args_info.profile_prefix_path);
+    return true;
+  }
+
+  fs::path new_profile_path;
   bool new_profile_use = false;
 
   if (util::starts_with(arg, "-fprofile-dir=")) {
@@ -267,7 +271,7 @@ process_profiling_option(const Context& ctx,
       new_profile_path = ".";
     } else {
       // GCC uses $PWD/$(basename $obj).
-      new_profile_path = pstr(ctx.apparent_cwd).str();
+      new_profile_path = ctx.apparent_cwd;
     }
   } else if (util::starts_with(arg, "-fprofile-generate=")
              || util::starts_with(arg, "-fprofile-instr-generate=")) {
@@ -395,10 +399,8 @@ process_option_arg(const Context& ctx,
     if (argpath[-1] == '-') {
       ++argpath;
     }
-    auto file_args = Args::from_atfile(argpath,
-                                       config.is_compiler_group_msvc()
-                                         ? Args::AtFileFormat::msvc
-                                         : Args::AtFileFormat::gcc);
+    auto file_args =
+      Args::from_response_file(argpath, config.response_file_format());
     if (!file_args) {
       LOG("Couldn't read arg file {}", argpath);
       return Statistic::bad_compiler_arguments;
@@ -421,7 +423,8 @@ process_option_arg(const Context& ctx,
     // Argument is a comma-separated list of files.
     auto paths = util::split_into_strings(args[i], ",");
     for (auto it = paths.rbegin(); it != paths.rend(); ++it) {
-      auto file_args = Args::from_atfile(*it);
+      auto file_args =
+        Args::from_response_file(*it, Args::ResponseFileFormat::posix);
       if (!file_args) {
         LOG("Couldn't read CUDA options file {}", *it);
         return Statistic::bad_compiler_arguments;
@@ -591,7 +594,7 @@ process_option_arg(const Context& ctx,
   }
 
   // We must have -c.
-  if (arg == "-c") {
+  if (arg == "-c" || arg == "--compile") { // --compile is NVCC
     state.found_c_opt = true;
     return Statistic::none;
   }
@@ -610,6 +613,18 @@ process_option_arg(const Context& ctx,
       state.input_files.emplace_back(arg.substr(3));
       return Statistic::none;
     }
+
+    if (arg == "-TC") {
+      args_info.actual_language = "c";
+      state.common_args.push_back(args[i]);
+      return Statistic::none;
+    }
+
+    if (arg == "-TP") {
+      args_info.actual_language = "c++";
+      state.common_args.push_back(args[i]);
+      return Statistic::none;
+    }
   }
 
   // when using nvcc with separable compilation, -dc implies -c
@@ -694,6 +709,29 @@ process_option_arg(const Context& ctx,
     return Statistic::none;
   }
 
+  if (util::starts_with(arg, "-fdebug-compilation-dir")
+      || util::starts_with(arg, "-ffile-compilation-dir")) {
+    std::string compilation_dir;
+    // -ffile-compilation-dir cannot be followed by a space.
+    if (arg == "-fdebug-compilation-dir") {
+      if (i == args.size() - 1) {
+        LOG("Missing argument to {}", args[i]);
+        return Statistic::bad_compiler_arguments;
+      }
+      state.common_args.push_back(args[i]);
+      compilation_dir = args[i + 1];
+      i++;
+    } else {
+      const auto eq_pos = arg.find('=');
+      if (eq_pos != std::string_view::npos) {
+        compilation_dir = arg.substr(eq_pos + 1);
+      }
+    }
+    args_info.compilation_dir = std::move(compilation_dir);
+    state.common_args.push_back(args[i]);
+    return Statistic::none;
+  }
+
   // Debugging is handled specially, so that we know if we can strip line
   // number info.
   if (util::starts_with(arg, "-g")) {
@@ -775,15 +813,14 @@ process_option_arg(const Context& ctx,
 
     if (state.output_dep_origin <= OutputDepOrigin::mf) {
       state.output_dep_origin = OutputDepOrigin::mf;
-      args_info.output_dep =
-        pstr(core::make_relative_path(ctx, dep_file)).str();
+      args_info.output_dep = core::make_relative_path(ctx, dep_file);
     }
     // Keep the format of the args the same.
     if (separate_argument) {
       state.dep_args.push_back("-MF");
       state.dep_args.push_back(args_info.output_dep);
     } else {
-      state.dep_args.push_back("-MF" + args_info.output_dep);
+      state.dep_args.push_back(FMT("-MF{}", args_info.output_dep));
     }
     return Statistic::none;
   }
@@ -833,7 +870,9 @@ process_option_arg(const Context& ctx,
     return Statistic::none;
   }
 
-  if (arg == "-showIncludes") {
+  if (arg == "-showIncludes"
+      // clang-cl:
+      || arg == "-showIncludes:user") {
     args_info.generating_includes = true;
     state.dep_args.push_back(args[i]);
     return Statistic::none;
@@ -886,6 +925,7 @@ process_option_arg(const Context& ctx,
       // the actual CWD in the .gcno file.
       state.hash_actual_cwd = true;
     }
+    state.common_args.push_back(args[i]);
     return Statistic::none;
   }
 
@@ -921,7 +961,7 @@ process_option_arg(const Context& ctx,
     }
     state.common_args.push_back(args[i]);
     auto relpath = core::make_relative_path(ctx, args[i + 1]);
-    state.common_args.push_back(pstr(relpath).str());
+    state.common_args.push_back(relpath);
     i++;
     return Statistic::none;
   }
@@ -1013,8 +1053,7 @@ process_option_arg(const Context& ctx,
       return Statistic::bad_compiler_arguments;
     }
     args_info.generating_diagnostics = true;
-    args_info.output_dia =
-      pstr(core::make_relative_path(ctx, args[i + 1])).str();
+    args_info.output_dia = core::make_relative_path(ctx, args[i + 1]);
     i++;
     return Statistic::none;
   }
@@ -1081,6 +1120,11 @@ process_option_arg(const Context& ctx,
     return Statistic::none;
   }
 
+  if (util::starts_with(arg, "-fbuild-session-file")
+      && !(config.sloppiness().contains(core::Sloppy::time_macros))) {
+    args_info.build_session_file = arg.substr(arg.find('=') + 1);
+  }
+
   if (config.sloppiness().contains(core::Sloppy::clang_index_store)
       && arg == "-index-store-path") {
     // Xcode 9 or later calls Clang with this option. The given path includes a
@@ -1129,7 +1173,7 @@ process_option_arg(const Context& ctx,
     if (next == 2) {
       dest_args.push_back(args[i + 1]);
     }
-    dest_args.push_back(pstr(relpath).str());
+    dest_args.push_back(relpath);
 
     i += next;
     return Statistic::none;
@@ -1152,17 +1196,16 @@ process_option_arg(const Context& ctx,
   // Potentially rewrite concatenated absolute path argument to relative.
   if (arg[0] == '-') {
     const auto [option, path] = util::split_option_with_concat_path(arg);
-    if (path) {
-      if (compopt_takes_concat_arg(option) && compopt_takes_path(option)) {
-        const auto relpath = core::make_relative_path(ctx, *path);
-        std::string new_option = FMT("{}{}", option, relpath);
-        if (compopt_affects_cpp_output(option)) {
-          state.cpp_args.push_back(std::move(new_option));
-        } else {
-          state.common_args.push_back(std::move(new_option));
-        }
-        return Statistic::none;
+    if (path && compopt_takes_concat_arg(option)
+        && compopt_takes_path(option)) {
+      const auto relpath = core::make_relative_path(ctx, *path);
+      std::string new_option = FMT("{}{}", option, relpath);
+      if (compopt_affects_cpp_output(option)) {
+        state.cpp_args.push_back(std::move(new_option));
+      } else {
+        state.common_args.push_back(std::move(new_option));
       }
+      return Statistic::none;
     }
   }
 
@@ -1300,7 +1343,7 @@ process_args(Context& ctx)
     if (is_link) {
       LOG_RAW("Called for link");
       return tl::unexpected(
-        pstr(state.input_files.front()).str().find("conftest.")
+        util::pstr(state.input_files.front()).str().find("conftest.")
             != std::string::npos
           ? Statistic::autoconf_test
           : Statistic::called_for_link);
@@ -1310,10 +1353,10 @@ process_args(Context& ctx)
     }
   }
 
-  args_info.orig_input_file = pstr(state.input_files.front()).str();
+  args_info.orig_input_file = state.input_files.front();
   // Rewrite to relative to increase hit rate.
   args_info.input_file =
-    pstr(core::make_relative_path(ctx, args_info.orig_input_file)).str();
+    core::make_relative_path(ctx, args_info.orig_input_file);
 
   // Bail out on too hard combinations of options.
   if (state.found_mf_opt && state.found_wp_md_or_mmd_opt) {
@@ -1342,10 +1385,9 @@ process_args(Context& ctx)
   // Determine output object file.
   bool output_obj_by_source = args_info.output_obj.empty();
   if (!output_obj_by_source && ctx.config.is_compiler_group_msvc()) {
-    if (*args_info.output_obj.rbegin() == '\\') {
+    if (util::pstr(args_info.output_obj).str().back() == '\\') {
       output_obj_by_source = true;
     } else if (DirEntry(args_info.output_obj).is_directory()) {
-      args_info.output_obj.append("\\");
       output_obj_by_source = true;
     }
   }
@@ -1359,22 +1401,19 @@ process_args(Context& ctx)
     } else {
       extension = get_default_object_file_extension(ctx.config);
     }
-    args_info.output_obj +=
-      pstr(
-        fs::path(args_info.input_file).filename().replace_extension(extension))
-        .str();
+    args_info.output_obj /= util::with_extension(
+      fs::path(args_info.input_file).filename(), extension);
   }
 
   args_info.orig_output_obj = args_info.output_obj;
-  args_info.output_obj =
-    pstr(core::make_relative_path(ctx, args_info.output_obj)).str();
+  args_info.output_obj = core::make_relative_path(ctx, args_info.output_obj);
 
   // Determine a filepath for precompiled header.
   if (ctx.config.is_compiler_group_msvc() && args_info.generating_pch) {
     bool included_pch_file_by_source = args_info.included_pch_file.empty();
 
     if (!included_pch_file_by_source
-        && (*args_info.orig_included_pch_file.rbegin() == '\\'
+        && (util::pstr(args_info.orig_included_pch_file).str().back() == '\\'
             || DirEntry(args_info.orig_included_pch_file).is_directory())) {
       LOG("Unsupported folder path value for -Fp: {}",
           args_info.included_pch_file);
@@ -1383,10 +1422,8 @@ process_args(Context& ctx)
 
     if (included_pch_file_by_source && !args_info.input_file.empty()) {
       args_info.included_pch_file =
-        pstr(fs::path(args_info.input_file)
-               .filename()
-               .replace_extension(get_default_pch_file_extension(ctx.config)))
-          .str();
+        util::with_extension(fs::path(args_info.input_file).filename(),
+                             get_default_pch_file_extension(ctx.config));
       LOG(
         "Setting PCH filepath from the base source file (during generating): "
         "{}",
@@ -1437,7 +1474,7 @@ process_args(Context& ctx)
   }
 
   if (args_info.profile_path.empty()) {
-    args_info.profile_path = pstr(ctx.apparent_cwd).str();
+    args_info.profile_path = ctx.apparent_cwd;
   }
 
   if (!state.explicit_language.empty() && state.explicit_language == "none") {
@@ -1449,7 +1486,7 @@ process_args(Context& ctx)
       return tl::unexpected(Statistic::unsupported_source_language);
     }
     args_info.actual_language = state.explicit_language;
-  } else {
+  } else if (args_info.actual_language.empty()) {
     args_info.actual_language =
       language_for_file(args_info.input_file, config.compiler_type());
   }
@@ -1459,10 +1496,10 @@ process_args(Context& ctx)
     || is_precompiled_header(args_info.output_obj);
 
   if (args_info.output_is_precompiled_header && output_obj_by_source) {
-    args_info.orig_output_obj =
-      args_info.orig_input_file + get_default_pch_file_extension(config);
+    args_info.orig_output_obj = util::add_extension(
+      args_info.orig_input_file, get_default_pch_file_extension(config));
     args_info.output_obj =
-      pstr(core::make_relative_path(ctx, args_info.orig_output_obj)).str();
+      core::make_relative_path(ctx, args_info.orig_output_obj);
   }
 
   if (args_info.output_is_precompiled_header
@@ -1480,10 +1517,11 @@ process_args(Context& ctx)
       LOG_RAW("No -c option found");
       // Having a separate statistic for autoconf tests is useful, as they are
       // the dominant form of "called for link" in many cases.
-      return tl::unexpected(args_info.input_file.find("conftest.")
-                                != std::string::npos
-                              ? Statistic::autoconf_test
-                              : Statistic::called_for_link);
+      return tl::unexpected(
+        util::pstr(args_info.input_file).str().find("conftest.")
+            != std::string::npos
+          ? Statistic::autoconf_test
+          : Statistic::called_for_link);
     }
   }
 
@@ -1524,8 +1562,7 @@ process_args(Context& ctx)
       // we haven't seen the -gsplit-dwarf option.
       args_info.seen_split_dwarf = false;
     } else {
-      args_info.output_dwo =
-        pstr(fs::path(args_info.output_obj).replace_extension(".dwo")).str();
+      args_info.output_dwo = util::with_extension(args_info.output_obj, ".dwo");
     }
   }
 
@@ -1541,7 +1578,7 @@ process_args(Context& ctx)
     args_info.generating_dependencies = false;
   }
 
-  fs::path output_dir = fs::path(args_info.output_obj).parent_path();
+  fs::path output_dir = args_info.output_obj.parent_path();
   if (!output_dir.empty() && !fs::is_directory(output_dir)) {
     LOG("Directory does not exist: {}", output_dir);
     return tl::unexpected(Statistic::bad_output_file);
@@ -1586,8 +1623,7 @@ process_args(Context& ctx)
 
   if (args_info.generating_dependencies) {
     if (state.output_dep_origin == OutputDepOrigin::none) {
-      args_info.output_dep =
-        pstr(fs::path(args_info.output_obj).replace_extension(".d")).str();
+      args_info.output_dep = util::with_extension(args_info.output_obj, ".d");
       if (!config.run_second_cpp()) {
         // If we're compiling preprocessed code we're sending dep_args to the
         // preprocessor so we need to use -MF to write to the correct .d file
@@ -1606,7 +1642,7 @@ process_args(Context& ctx)
     }
 
     if (!args_info.dependency_target) {
-      std::string dep_target = args_info.orig_output_obj;
+      fs::path dep_target = args_info.orig_output_obj;
 
       // GCC and Clang behave differently when "-Wp,-M[M]D,wp.d" is used with
       // "-o" but with neither "-MMD" nor "-MT"/"-MQ": GCC uses a dependency
@@ -1621,10 +1657,8 @@ process_args(Context& ctx)
           // GCC strangely uses the base name of the source file but with a .o
           // extension.
           dep_target =
-            fs::path(args_info.orig_input_file)
-              .filename()
-              .replace_extension(get_default_object_file_extension(ctx.config))
-              .string();
+            util::with_extension(args_info.orig_input_file.filename(),
+                                 get_default_object_file_extension(ctx.config));
         } else {
           // How other compilers behave is currently unknown, so bail out.
           LOG_RAW(
@@ -1634,27 +1668,26 @@ process_args(Context& ctx)
         }
       }
 
-      args_info.dependency_target = Depfile::escape_filename(dep_target);
+      args_info.dependency_target =
+        Depfile::escape_filename(util::pstr(dep_target).str());
     }
   }
 
   if (args_info.generating_stackusage) {
-    std::string default_sufile_name =
-      fs::path(args_info.output_obj).replace_extension(".su").string();
-    args_info.output_su =
-      pstr(core::make_relative_path(ctx, default_sufile_name)).str();
+    fs::path default_sufile_name =
+      util::with_extension(args_info.output_obj, ".su");
+    args_info.output_su = core::make_relative_path(ctx, default_sufile_name);
   }
 
   if (args_info.generating_callgraphinfo) {
-    std::string default_cifile_name =
-      fs::path(args_info.output_obj).replace_extension(".ci").string();
-    args_info.output_ci =
-      pstr(core::make_relative_path(ctx, default_cifile_name)).str();
+    fs::path default_cifile_name =
+      util::with_extension(args_info.output_obj, ".ci");
+    args_info.output_ci = core::make_relative_path(ctx, default_cifile_name);
   }
 
   if (args_info.generating_ipa_clones) {
-    fs::path ipa_path(args_info.orig_input_file + ".000i.ipa-clones");
-    args_info.output_ipa = pstr(core::make_relative_path(ctx, ipa_path)).str();
+    args_info.output_ipa = core::make_relative_path(
+      ctx, util::add_extension(args_info.orig_input_file, ".000i.ipa-clones"));
   }
 
   Args compiler_args = state.common_args;
@@ -1766,11 +1799,11 @@ process_args(Context& ctx)
 }
 
 bool
-is_precompiled_header(std::string_view path)
+is_precompiled_header(const fs::path& path)
 {
-  fs::path ext = fs::path(path).extension();
+  fs::path ext = path.extension();
   return ext == ".gch" || ext == ".pch" || ext == ".pth"
-         || fs::path(path).parent_path().extension() == ".gch";
+         || path.parent_path().extension() == ".gch";
 }
 
 bool
