@@ -1,4 +1,4 @@
-// Copyright (C) 2021-2024 Joel Rosdahl and other contributors
+// Copyright (C) 2021-2025 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -18,39 +18,39 @@
 
 #include "mainoptions.hpp"
 
-#include <ccache/Config.hpp>
-#include <ccache/Hash.hpp>
-#include <ccache/InodeCache.hpp>
-#include <ccache/ProgressBar.hpp>
 #include <ccache/ccache.hpp>
-#include <ccache/core/CacheEntry.hpp>
-#include <ccache/core/FileRecompressor.hpp>
-#include <ccache/core/Manifest.hpp>
-#include <ccache/core/Result.hpp>
-#include <ccache/core/ResultExtractor.hpp>
-#include <ccache/core/ResultInspector.hpp>
-#include <ccache/core/Statistics.hpp>
-#include <ccache/core/StatsLog.hpp>
+#include <ccache/config.hpp>
+#include <ccache/core/cacheentry.hpp>
 #include <ccache/core/exceptions.hpp>
-#include <ccache/storage/Storage.hpp>
-#include <ccache/storage/local/LocalStorage.hpp>
-#include <ccache/util/Fd.hpp>
-#include <ccache/util/FileStream.hpp>
-#include <ccache/util/PathString.hpp>
-#include <ccache/util/TemporaryFile.hpp>
-#include <ccache/util/TextTable.hpp>
-#include <ccache/util/ThreadPool.hpp>
-#include <ccache/util/UmaskScope.hpp>
-#include <ccache/util/XXH3_128.hpp>
+#include <ccache/core/filerecompressor.hpp>
+#include <ccache/core/manifest.hpp>
+#include <ccache/core/result.hpp>
+#include <ccache/core/resultextractor.hpp>
+#include <ccache/core/resultinspector.hpp>
+#include <ccache/core/statistics.hpp>
+#include <ccache/core/statslog.hpp>
+#include <ccache/hash.hpp>
+#include <ccache/inodecache.hpp>
+#include <ccache/progressbar.hpp>
+#include <ccache/storage/local/localstorage.hpp>
+#include <ccache/storage/storage.hpp>
 #include <ccache/util/assertions.hpp>
 #include <ccache/util/cpu.hpp>
 #include <ccache/util/environment.hpp>
 #include <ccache/util/expected.hpp>
+#include <ccache/util/fd.hpp>
 #include <ccache/util/file.hpp>
+#include <ccache/util/filestream.hpp>
 #include <ccache/util/filesystem.hpp>
 #include <ccache/util/format.hpp>
 #include <ccache/util/logging.hpp>
+#include <ccache/util/path.hpp>
 #include <ccache/util/string.hpp>
+#include <ccache/util/temporaryfile.hpp>
+#include <ccache/util/texttable.hpp>
+#include <ccache/util/threadpool.hpp>
+#include <ccache/util/umaskscope.hpp>
+#include <ccache/util/xxh3_128.hpp>
 
 #include <fcntl.h>
 
@@ -76,7 +76,6 @@
 namespace fs = util::filesystem;
 
 using util::DirEntry;
-using pstr = util::PathString;
 
 namespace core {
 
@@ -85,7 +84,7 @@ constexpr const char VERSION_TEXT[] =
 Features: {2}
 
 Copyright (C) 2002-2007 Andrew Tridgell
-Copyright (C) 2009-2024 Joel Rosdahl and other contributors
+Copyright (C) 2009-2025 Joel Rosdahl and other contributors
 
 See <https://ccache.dev/credits.html> for a complete list of contributors.
 
@@ -195,7 +194,7 @@ configuration_printer(const std::string& key,
 }
 
 static tl::expected<util::Bytes, std::string>
-read_from_path_or_stdin(const std::string& path)
+read_from_path_or_stdin(const fs::path& path)
 {
   if (path == "-") {
     return util::read_fd(STDIN_FILENO).transform_error([&](auto error) {
@@ -208,7 +207,7 @@ read_from_path_or_stdin(const std::string& path)
 }
 
 static int
-inspect_path(const std::string& path)
+inspect_path(const fs::path& path)
 {
   std::optional<util::DirEntry> orig_dir_entry;
   if (path != "-") {
@@ -227,7 +226,7 @@ inspect_path(const std::string& path)
   }
 
   core::CacheEntry cache_entry(*cache_entry_data);
-  fputs(cache_entry.header().inspect().c_str(), stdout);
+  std::ignore = fputs(cache_entry.header().inspect().c_str(), stdout);
 
   const auto payload = cache_entry.payload();
 
@@ -562,7 +561,7 @@ process_main_options(int argc, const char* const* argv)
     }
 
     case TRIM_METHOD:
-      trim_lru_mtime = (arg == "ctime");
+      trim_lru_mtime = (arg == "mtime");
       break;
 
     case TRIM_RECOMPRESS:
@@ -721,7 +720,7 @@ process_main_options(int argc, const char* const* argv)
     case 'F': { // --max-files
       auto files = util::value_or_throw<Error>(util::parse_unsigned(arg));
       config.set_value_in_file(
-        pstr(config.config_path()).str(), "max_files", arg);
+        util::pstr(config.config_path()), "max_files", arg);
       if (files == 0) {
         PRINT_RAW(stdout, "Unset cache file limit\n");
       } else {
@@ -735,7 +734,7 @@ process_main_options(int argc, const char* const* argv)
         util::value_or_throw<Error>(util::parse_size(arg));
       uint64_t max_size = size;
       config.set_value_in_file(
-        pstr(config.config_path()).str(), "max_size", arg);
+        util::pstr(config.config_path()), "max_size", arg);
       if (max_size == 0) {
         PRINT_RAW(stdout, "Unset cache size limit\n");
       } else {
@@ -755,7 +754,7 @@ process_main_options(int argc, const char* const* argv)
       }
       std::string key = arg.substr(0, eq_pos);
       std::string value = arg.substr(eq_pos + 1);
-      config.set_value_in_file(pstr(config.config_path()).str(), key, value);
+      config.set_value_in_file(util::pstr(config.config_path()), key, value);
       break;
     }
 
@@ -812,7 +811,8 @@ process_main_options(int argc, const char* const* argv)
 
     case 'V': // --version
     {
-      PRINT_RAW(stdout, get_version_text(pstr(fs::path(argv[0]).stem()).str()));
+      PRINT_RAW(stdout,
+                get_version_text(util::pstr(fs::path(argv[0]).stem()).str()));
       break;
     }
 
